@@ -5,7 +5,7 @@
 #' @param n_variants Integer, number of model variants per horizon (ensemble size).
 #' @param engines Character vector of supported mean-model engines ('rpart', 'glmnet', 'knn').
 #' @param dists Character vector of supported residual distributions (gaussian, laplace, student, logistic, asymmetric_laplace, skew_normal, skew_t.
-#' @param h_options Character vector for supported heteroscedastic scale models (tree, ridge, glmnet).
+#' @param h_options Character vector for supported heteroscedastic scale models (tree, ridge).
 #' @param alpha,beta Numeric weights combining CRPS and calibration error.
 #' @param temperature Softmax temperature for ensemble weighting (>0).
 #' @param dates Vector for date formats. Default: NULL.
@@ -55,7 +55,7 @@
 organik <- function(ts, horizon, n_variants = 10,
                     engines = c("rpart","glmnet","knn"),
                     dists   = c("gaussian","laplace","student","logistic","asymmetric_laplace","skew_normal","skew_t"),
-                    h_options = c("tree", "ridge", "glmnet"),
+                    h_options = c("tree", "ridge"),
                     alpha = 1.0, beta = 1.0, temperature = 1,
                     dates = NULL, ci = 0.95,
                     n_testing = 30,
@@ -101,25 +101,26 @@ organik <- function(ts, horizon, n_variants = 10,
   names(level_pred_funs) <- paste0("t", seq_len(horizon))
 
   # ---------- Empirical correlation on cumulative targets ----------
-  cor_mat <- suppressWarnings(cor(y_mat, use = "pairwise.complete.obs", method = "spearman"))
-  cor_mat[!is.finite(cor_mat)] <- 0
-  cor_mat <- 2 * sin(cor_mat * pi/6)
-  diag(cor_mat) <- 1
+  #cor_mat <- suppressWarnings(cor(y_mat, use = "pairwise.complete.obs", method = "spearman"))
+  #cor_mat[!is.finite(cor_mat)] <- 0
+  #cor_mat <- 2 * sin(cor_mat * pi/6)
+  #diag(cor_mat) <- 1
 
-  ensure_pd <- function(S) {
-    pd_ok <- FALSE
-    eig <- tryCatch(eigen(S, symmetric = TRUE, only.values = TRUE)$values, error = function(e) NA)
-    if (all(is.finite(eig)) && min(eig) > 1e-8) pd_ok <- TRUE
-    if (pd_ok) return(S)
-    if (requireNamespace("Matrix", quietly = TRUE)) {
-      S2 <- as.matrix(nearPD(S, corr = TRUE)$mat)
-      return(S2)
-    } else {
-      eps <- 1e-6
-      return(S + diag(eps, nrow(S)))
+  t_corr_from_kendall <- function(Y) {
+    Tau <- suppressWarnings(stats::cor(Y, method = "kendall", use = "pairwise.complete.obs"))
+    R   <- sin(pi * Tau / 2)
+    diag(R) <- 1
+    R <- (R + t(R)) / 2
+    ensure_pd <- function(S) {
+      ev <- tryCatch(eigen(S, symmetric = TRUE, only.values = TRUE)$values, error = function(e) NA)
+      if (all(is.finite(ev)) && min(ev) > 1e-8) return(S)
+      if (requireNamespace("Matrix", quietly = TRUE)) as.matrix(Matrix::nearPD(S, corr = TRUE)$mat)
+      else S + diag(1e-6, nrow(S))
     }
+    ensure_pd(R)
   }
-  cor_mat <- ensure_pd(cor_mat)
+
+  cor_mat <- t_corr_from_kendall(y_mat)
 
   # ---------- Joint path prediction via copula ----------
   path_prediction <- function(n_paths = 5000,
@@ -223,7 +224,7 @@ ensemble <- function(
     n_variants = 12,
     engines = c("rpart","glmnet","knn"),
     dists   = c("gaussian","laplace","student","logistic","asymmetric_laplace","skew_normal","skew_t"),
-    h_options = c("ridge","tree","glmnet"),
+    h_options = c("ridge","tree"),
     alpha = 1.0, beta = 1.0, temperature = 0.25,
     n_testing = 50, mc_crps_n = 1000, cap_to_y = TRUE,
     max_warn_retries = 2,
@@ -285,12 +286,6 @@ ensemble <- function(
                                ", maxdepth=", fmt_num(tc$maxdepth),
                                ", usesurrogate=", fmt_num(tc$usesurrogate), "}")
                       },
-                      "glmnet" = {
-                        lam <- tryCatch(model_obj$hetero_fit$lambda, error = function(e) NA_real_)
-                        paste0("hetero=glmnet{alpha=", fmt_num(cfg$h_params$alpha),
-                               ", lambda=", fmt_num(lam),
-                               ", standardize=", cfg$h_params$standardize %||% TRUE, "}")
-                      },
                       paste0("hetero=", cfg$hetero)
     )
 
@@ -370,11 +365,6 @@ ensemble <- function(
                           maxdepth  = sample(2:9, 1),
                           usesurrogate = sample(0:2, 1)
                         )
-                      ),
-                      "glmnet" = list(
-                        alpha       = runif(1, 0, 1),
-                        lambda      = NULL,
-                        standardize = TRUE
                       )
     )
 
@@ -434,9 +424,8 @@ ensemble <- function(
     variants[[i]] <- model_i
 
     bt <- model_i$backtest
-    mean_crsp <- safe_mean(bt$crsp)
+    mean_crsp <-safe_mean(bt$crsp)
     cal_last  <- tail(bt$calibration_error, 1)
-    score     <- 1 / (alpha * mean_crsp + beta * cal_last)
 
     meta[[i]] <- data.frame(
       id = i,
@@ -445,7 +434,6 @@ ensemble <- function(
       hetero = cfg_used$hetero,
       mean_crsp = mean_crsp,
       calibration_error = cal_last,
-      score = score,
       specific_params = summarize_cfg(cfg_used, model_i),
       stringsAsFactors = FALSE
     )
@@ -453,6 +441,13 @@ ensemble <- function(
 
   meta_df <- do.call(rbind, meta)
 
+  #minmax <- function(x) (x - min(x))/(max(x) - min(x))
+
+  scaled_crps <- as.vector(-scale(meta_df$mean_crsp))
+  scaled_cals <- as.vector(-scale(meta_df$calibration_error))
+  meta_df$score <- alpha * scaled_crps + beta * scaled_cals
+
+
   softmax <- function(v, temperature = 1)
   {
     if (any(!is.finite(v))) {v[!is.finite(v)] <- mean(v[is.finite(v)], na.rm = TRUE) + 3 * sd(v[is.finite(v)], na.rm = TRUE)}
@@ -470,65 +465,106 @@ ensemble <- function(
   make_mixture_predfun <- function(pfuns_list, weights, level = NULL,
                                    lo_q = 1e-4, hi_q = 1 - 1e-4,
                                    n_grid = 2000, max_expand = 6L) {
-    k <- length(pfuns_list); w <- as.numeric(weights)
+    k <- length(pfuns_list)
+    w <- as.numeric(weights)
 
+    # Mixture density / CDF defined on:
+    #  - growth scale if level is NULL
+    #  - level scale if level is not NULL (internally mapped to growth)
     dfun <- function(x) {
-      if(!is.null(level)){x <- (x/level) - 1}
+      if (!is.null(level)) {
+        x <- (x / level) - 1  # map level -> growth
+      }
       x <- as.numeric(x)
       dens_mat <- vapply(pfuns_list, function(pf) pf$dfun(x), numeric(length(x)))
       as.numeric(dens_mat %*% w)
     }
 
     pfun <- function(q) {
-      if(!is.null(level)){q <- (q/level) - 1}
+      if (!is.null(level)) {
+        q <- (q / level) - 1  # map level -> growth
+      }
       q <- as.numeric(q)
-      cdf_mat  <- vapply(pfuns_list, function(pf) pf$pfun(q), numeric(length(q)))
+      cdf_mat <- vapply(pfuns_list, function(pf) pf$pfun(q), numeric(length(q)))
       as.numeric(cdf_mat %*% w)
     }
 
+    # Support bounds first on growth scale
     q_los <- vapply(pfuns_list, function(pf) pf$qfun(lo_q), numeric(1))
     q_his <- vapply(pfuns_list, function(pf) pf$qfun(hi_q), numeric(1))
-    lo <- min(q_los); hi <- max(q_his)
-    if (!is.finite(lo) || !is.finite(hi) || !(hi > lo)) {
+    lo_g <- min(q_los)
+    hi_g <- max(q_his)
+    if (!is.finite(lo_g) || !is.finite(hi_g) || !(hi_g > lo_g)) {
       mu_guess <- vapply(pfuns_list, function(pf) pf$qfun(0.5), numeric(1))
-      lo <- min(mu_guess) - 10; hi <- max(mu_guess) + 10
+      lo_g <- min(mu_guess) - 10
+      hi_g <- max(mu_guess) + 10
+    }
+
+    # Map support to level scale if needed
+    if (is.null(level)) {
+      lower <- lo_g
+      upper <- hi_g
+    } else {
+      lower <- level * (1 + lo_g)
+      upper <- level * (1 + hi_g)
     }
 
     make_cdf_qfun_safe <- function(pdf, lower, upper, n_grid, max_expand = 6L) {
       expand <- 0L
       repeat {
-        x_grid <- seq(lower, upper, length.out = n_grid); dx <- diff(x_grid)[1]
-        dens <- pdf(x_grid); dens[!is.finite(dens)] <- 0; dens <- pmax(dens, 0)
+        x_grid <- seq(lower, upper, length.out = n_grid)
+        dx <- diff(x_grid)[1]
+        dens <- pdf(x_grid)
+        dens[!is.finite(dens)] <- 0
+        dens <- pmax(dens, 0)
         cdf_vals <- c(0, cumsum((dens[-1] + dens[-length(dens)]) * 0.5) * dx)
         mass <- cdf_vals[length(cdf_vals)]
         if (!is.finite(mass) || mass <= 0) {
-          if (expand < max_expand) { expand <- expand + 1L; rng <- (upper - lower) * 0.5; lower <- lower - rng; upper <- upper + rng; next }
-          else { cdf_vals <- seq(0, 1, length.out = length(x_grid));
-          cdf_fun  <- suppressWarnings(approxfun(x_grid, cdf_vals, yleft = 0, yright = 1, rule = 2));
-          qfun_fun <- suppressWarnings(approxfun(cdf_vals, x_grid, yleft = lower, yright = upper, rule = 2));
-          return(list(cdf = cdf_fun, qfun = qfun_fun)) }
-        } else cdf_vals <- cdf_vals / mass
-        cdf_vals <- pmin(pmax(cdf_vals, 0), 1); cdf_vals <- cummax(cdf_vals)
-        if (length(unique(cdf_vals)) < 2L) {
-          if (expand < max_expand) { expand <- expand + 1L; rng <- (upper - lower) * 0.5; lower <- lower - rng; upper <- upper + rng; next }
-          else { cdf_vals <- seq(0, 1, length.out = length(x_grid)) }
+          if (expand < max_expand) {
+            expand <- expand + 1L
+            rng <- (upper - lower) * 0.5
+            lower <- lower - rng
+            upper <- upper + rng
+            next
+          } else {
+            cdf_vals <- seq(0, 1, length.out = length(x_grid))
+            cdf_fun  <- approxfun(x_grid, cdf_vals, yleft = 0, yright = 1, rule = 2)
+            qfun_fun <- approxfun(cdf_vals, x_grid, yleft = lower, yright = upper, rule = 2)
+            return(list(cdf = cdf_fun, qfun = qfun_fun))
+          }
         } else {
-          jitter <- seq(0, (length(cdf_vals)-1)) * 1e-14
-          cdf_vals <- pmin(pmax(cdf_vals + jitter, 0), 1)
-          cdf_vals <- (cdf_vals - min(cdf_vals)) / max(1e-12, (max(cdf_vals) - min(cdf_vals)))
+          cdf_vals <- cdf_vals / mass
         }
-        cdf_fun  <- suppressWarnings(approxfun(x_grid, cdf_vals, yleft = 0, yright = 1, rule = 2))
-        qfun_fun <- suppressWarnings(approxfun(cdf_vals, x_grid, yleft = lower, yright = upper, rule = 2))
+        cdf_vals <- pmin(pmax(cdf_vals, 0), 1)
+        cdf_vals <- cummax(cdf_vals)
+        if (length(unique(cdf_vals)) < 2L) {
+          if (expand < max_expand) {
+            expand <- expand + 1L
+            rng <- (upper - lower) * 0.5
+            lower <- lower - rng
+            upper <- upper + rng
+            next
+          } else {
+            cdf_vals <- seq(0, 1, length.out = length(x_grid))
+          }
+        } else {
+          jitter <- seq(0, (length(cdf_vals) - 1)) * 1e-14
+          cdf_vals <- pmin(pmax(cdf_vals + jitter, 0), 1)
+          cdf_vals <- (cdf_vals - min(cdf_vals)) /
+            max(1e-12, (max(cdf_vals) - min(cdf_vals)))
+        }
+        cdf_fun  <- approxfun(x_grid, cdf_vals, yleft = 0, yright = 1, rule = 2)
+        qfun_fun <- approxfun(cdf_vals, x_grid, yleft = lower, yright = upper, rule = 2)
         return(list(cdf = cdf_fun, qfun = qfun_fun))
       }
     }
 
-    grid <- make_cdf_qfun_safe(function(xx) dfun(xx), lo, hi, n_grid, max_expand)
+    # Grid is now on growth scale (level = NULL) or level scale (level != NULL),
+    # consistent with dfun()'s interpretation.
+    grid <- make_cdf_qfun_safe(function(xx) dfun(xx), lower, upper, n_grid, max_expand)
 
     qfun <- function(p) {
-      q <- grid$qfun(clamp01(p))
-      if(!is.null(level)) { q <- level * (1 + q) }
-      q
+      grid$qfun(clamp01(p))  # no extra level*(1+...) here
     }
 
     rfun <- function(n) {
@@ -538,7 +574,9 @@ ensemble <- function(
         idx <- which(comp == j)
         if (length(idx)) out[idx] <- pfuns_list[[j]]$rfun(length(idx))
       }
-      if(!is.null(level)) { out <- level * (1 + out) }
+      if (!is.null(level)) {
+        out <- level * (1 + out)  # map growth -> level
+      }
       out
     }
 
@@ -569,7 +607,7 @@ ensemble <- function(
     n_models    = n_variants,
     variants    = variants,
     weights     = weights,
-    leaderboard = meta_df[order(meta_df$score), ],
+    leaderboard = meta_df[order(-meta_df$score), ],
     predict     = predictor
   )
   attr(out$leaderboard, "engine_counts") <- engine_counts
@@ -664,62 +702,6 @@ build_hetero_ridge <- function(X, proxy, params, seed = NULL) {
 #'
 #' @keywords internal
 #'
-build_hetero_glmnet <- function(X, proxy, params, seed = NULL) {
-  # Preconditions & fallbacks
-  if (!requireNamespace("glmnet", quietly = TRUE))
-    stop("glmnet not available")
-
-  proxy <- as.numeric(proxy); proxy[!is.finite(proxy)] <- mean(proxy[is.finite(proxy)], na.rm = TRUE)
-  if (!length(proxy) || all(!is.finite(proxy))) proxy <- rep(0, nrow(X))
-
-  # Standardize & drop zero-variance cols
-  ss <- safe_standardize(X); Xs <- ss$Xs
-  n <- nrow(Xs); p <- ncol(Xs)
-  if (n < 3L) stop("too few rows for glmnet")
-  if (p == 0L) stop("no predictors for glmnet")
-
-  alpha       <- params$alpha       %||% 0.5
-  lambda_fix  <- params$lambda      %||% NULL
-  standardize <- params$standardize %||% TRUE
-  nfolds      <- min(max(3L, params$ridge_kfold %||% 5), n)  # reuse kfold param if present
-
-  # Fit cv.glmnet on proxy (log-scale target already)
-  set.seed(seed %||% 1L)
-  fit <- tryCatch({
-    cv.glmnet(
-      x = if (standardize) Xs else as.matrix(X[, ss$keep, drop = FALSE]),
-      y = proxy,
-      family = "gaussian",
-      alpha = alpha,
-      lambda = lambda_fix,
-      nfolds = nfolds,
-      intercept = TRUE,
-      standardize = FALSE  # we already standardized
-    )
-  }, error = function(e) e)
-
-  if (inherits(fit, "error")) stop(conditionMessage(fit))
-
-  chosen_lambda <- if (!is.null(lambda_fix)) lambda_fix else fit$lambda.min
-  # Closure uses glmnet's predict in log-scale then exp()
-  scale_fun <- function(Xnew) {
-    Xnew <- as.matrix(Xnew)
-    if (length(ss$keep) == 0L) {
-      const_s <- pmax(exp(mean(proxy, na.rm = TRUE)), 1e-8)
-      return(rep(const_s, nrow(Xnew)))
-    }
-    Xk <- Xnew[, ss$keep, drop = FALSE]
-    Xk <- if (standardize) scale(Xk, center = ss$mu, scale = ss$sd) else Xk
-    log_s <- drop(predict(fit, newx = Xk, s = chosen_lambda, type = "response"))
-    pmax(exp(log_s), 1e-8)
-  }
-
-  list(scale_fun = scale_fun,
-       fit_info  = list(method = "glmnet", alpha = alpha, lambda = chosen_lambda))
-}
-#'
-#' @keywords internal
-#'
 fit_ridge <- function(X, y, lambda, standardize = TRUE) {
 X <- as.matrix(X); y <- as.numeric(y)
 n <- nrow(X); p <- ncol(X)
@@ -773,7 +755,7 @@ component_fit <- function(
     ),
     dist = c("gaussian","laplace","student","logistic","asymmetric_laplace","skew_normal","skew_t"),
     dist_params = list(nu = 5, tau = 0.5, alpha = 2.0),
-    hetero = c("ridge","none","tree","glmnet"),
+    hetero = c("ridge","none","tree"),
     h_params = list(
       # ridge
       ridge_lambda      = NULL,
@@ -781,11 +763,7 @@ component_fit <- function(
       ridge_kfold       = 5,
       standardize       = TRUE,
       # tree
-      tree_control      = NULL,
-      # glmnet
-      alpha       = 0.5,
-      lambda      = NULL,
-      standardize_glmnet = TRUE
+      tree_control      = NULL
     ),
     backtest  = TRUE,
     n_testing = 30,
@@ -942,25 +920,6 @@ component_fit <- function(
                             }
                           },
 
-                          "glmnet" = {
-                            out <- tryCatch(
-                              build_hetero_glmnet(X, proxy, h_params, seed),
-                              error = function(e) {
-                                # fallback to ridge
-                                tryCatch(build_hetero_ridge(X, proxy, h_params, seed),
-                                         error = function(e2) NULL)
-                              }
-                            )
-                            if (is.null(out)) {
-                              # final fallback: constant scale
-                              const_s <- pmax(exp(mean(proxy, na.rm = TRUE)), 1e-8)
-                              function(Xnew) rep(const_s, nrow(as.matrix(Xnew)))
-                            } else {
-                              hetero_fit <- out$fit_info  # store alpha/lambda (or ridge λ)
-                              out$scale_fun
-                            }
-                          },
-
                           "tree" = {
                             if (!requireNamespace("rpart", quietly = TRUE)) stop("Install 'rpart' for hetero='tree'.")
                             dfh <- data.frame(proxy = proxy, X)
